package com.example.afrishop_v3.controllers;

import com.example.afrishop_v3.models.*;
import com.example.afrishop_v3.repository.*;
import com.example.afrishop_v3.security.services.AuthenticationUser;
import org.springframework.data.domain.PageRequest;
import org.springframework.web.bind.annotation.*;

import javax.persistence.EntityManager;
import javax.persistence.Query;
import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.time.LocalDate;
import java.time.ZoneId;
import java.util.*;

@RestController
@RequestMapping("/discover/bonus")
public class BonusController extends Controller {
    private final BonusRepository repository;
    private final ItemRepository itemRepository;
    private final UserRepository userRepository;
    private final PostRepository postRepository;
    private final NetworkRepository networkRepository;
    private final EntityManager entityManager;
    private final AuthenticationUser user;

    public BonusController(BonusRepository repository, ItemRepository itemRepository, UserRepository userRepository, PostRepository postRepository, NetworkRepository networkRepository, EntityManager entityManager, AuthenticationUser user) {
        this.repository = repository;
        this.itemRepository = itemRepository;
        this.userRepository = userRepository;
        this.postRepository = postRepository;
        this.networkRepository = networkRepository;
        this.entityManager = entityManager;
        this.user = user;
    }

    @GetMapping(value = "/listBonus")
    //@PreAuthorize("hasAuthority('ADMIN_USER') or hasAuthority('STANDARD_USER')")
    public List<Bonus> getList(@RequestParam(value = "pageNo") Integer pageNo, @RequestParam(value = "pageSize") Integer pageSize) {
        return repository.findAll(PageRequest.of(pageNo, pageSize)).toList();
    }

    @GetMapping(value = "/list/bonus/{userId}")
    //@PreAuthorize("hasAuthority('ADMIN_USER') or hasAuthority('STANDARD_USER')")
    public List<Bonus> getListByUser(@PathVariable("userId") String id, @RequestParam(value = "pageNo") Integer pageNo, @RequestParam(value = "pageSize") Integer pageSize) {
        return repository.findAllByUserInfo_UserIdOrderByIdDesc(id, PageRequest.of(pageNo, pageSize)).toList();
    }

    @GetMapping(value = "/list/bonus/{userId}/currentMonth")
    //@PreAuthorize("hasAuthority('ADMIN_USER') or hasAuthority('STANDARD_USER')")
    public Map<String, Object> getListByUserAndCurrentMonth(@PathVariable("userId") String id, @RequestParam(value = "pageNo") Integer pageNo, @RequestParam(value = "pageSize") Integer pageSize) {
        Date date = new Date();
        LocalDate localDate = date.toInstant().atZone(ZoneId.systemDefault()).toLocalDate();
        int month = localDate.getMonthValue();
        int year = localDate.getYear();
        Optional<TbCfUserInfo> byId = userRepository.findById(id);
        if (!byId.isPresent()) return null;
        Query nativeQuery = entityManager.createNativeQuery("select IFNULL(sum(b.amount),0) as data FROM bonus b where b.user_info_user_id=:user and month (b.create_date) = :month and year(b.create_date) = :year ");
        nativeQuery.setParameter("user", id);
        nativeQuery.setParameter("month", month);
        nativeQuery.setParameter("year", year);
        List<Bonus> bonuses = repository.findAllByUser_IdAndCreateDateMonthAndCreateDateYear(byId.get(), month, year, PageRequest.of(pageNo, pageSize)).toList();
        HashMap<String, Object> hashMap = new HashMap<>();
        hashMap.put("total", nativeQuery.getSingleResult());
        hashMap.put("list", bonuses);
        return hashMap;
    }

    @GetMapping(value = "/list/bonus/{userId}/prevMonth")
    //@PreAuthorize("hasAuthority('ADMIN_USER') or hasAuthority('STANDARD_USER')")
    public Map<String, Object> getListByUserAndPreviousMonth(@PathVariable("userId") String id, @RequestParam(value = "pageNo") Integer pageNo, @RequestParam(value = "pageSize") Integer pageSize) {
        Date date = new Date();
        LocalDate localDate = date.toInstant().atZone(ZoneId.systemDefault()).toLocalDate();
        int month = localDate.getMonthValue() - 1;
        int year = localDate.getYear();
        Optional<TbCfUserInfo> byId = userRepository.findById(id);
        if (!byId.isPresent()) return null;
        Query nativeQuery = entityManager.createNativeQuery("select IFNULL(sum(b.amount),0) as data FROM bonus b where b.user_info_user_id=:user and month (b.create_date) = :month and year(b.create_date) = :year ");
        nativeQuery.setParameter("user", id);
        nativeQuery.setParameter("month", month);
        nativeQuery.setParameter("year", year);
        List<Bonus> bonuses = repository.findAllByUser_IdAndCreateDateMonthAndCreateDateYear(byId.get(), month, year, PageRequest.of(pageNo, pageSize)).toList();
        HashMap<String, Object> hashMap = new HashMap<>();
        hashMap.put("total", nativeQuery.getSingleResult());
        hashMap.put("list", bonuses);
        return hashMap;
    }

    @GetMapping(value = "/list/bonus/{userId}/today")
    //@PreAuthorize("hasAuthority('ADMIN_USER') or hasAuthority('STANDARD_USER')")
    public Map<String, Object> getListByUserAndToday(@PathVariable("userId") String id, @RequestParam(value = "pageNo") Integer pageNo, @RequestParam(value = "pageSize") Integer pageSize) {
        Date date = new Date();
        LocalDate localDate = date.toInstant().atZone(ZoneId.systemDefault()).toLocalDate();
        int dayOfYear = localDate.getDayOfYear();
        int year = localDate.getYear();
        Optional<TbCfUserInfo> byId = userRepository.findById(id);
        if (!byId.isPresent()) return null;
        Query nativeQuery = entityManager.createNativeQuery("select IFNULL(sum(b.amount),0) as data FROM bonus b where b.user_info_user_id=:user and dayofyear(b.create_date) = :day and year(b.create_date) = :year ");
        nativeQuery.setParameter("user", id);
        nativeQuery.setParameter("day", dayOfYear);
        nativeQuery.setParameter("year", year);
        List<Bonus> bonuses = repository.findAllByUser_IdAndCreateDate(byId.get(), dayOfYear, year, PageRequest.of(pageNo, pageSize)).toList();
        HashMap<String, Object> hashMap = new HashMap<>();
        hashMap.put("total", nativeQuery.getSingleResult());
        hashMap.put("list", bonuses);
        return hashMap;
    }

    @PostMapping(value = "/saveBonus")
    public Bonus saveBonus(@ModelAttribute("Bonus") Bonus bonus) {
        String itemId = bonus.getProduct();
        boolean exists = itemRepository.existsByItemName(itemId);
        Item byItemName = itemRepository.findByItemName(itemId);
        Post post = bonus.getPost();
        if (exists) {
            bonus.setItem(byItemName);
        } else return null;


        String sharer = bonus.getSharer();
        double amount = bonus.getAmount();
        bonus.setAmount(amount * 5 / 100);
        bonus.setItem(byItemName);
        bonus.setPercentage(5);

        if (sharer != null && userRepository.existsByCode(sharer)) {
            Optional<TbCfUserInfo> byId = userRepository.findByCode(sharer);
            if (byId.isPresent()) {
                Bonus bonus1 = new Bonus();
                bonus1.setItem(byItemName);
                bonus1.setPost(post);
                bonus1.setUserInfo(byId.get());
                bonus1.setAmount(amount * 10 / 100);
                bonus1.setPercentage(10.0);
                repository.save(bonus1);
            }
        }


        return repository.save(bonus);
    }

    @PostMapping(value = "/saveNetworkMarketing")
    public Bonus saveNetworkMarketing(@ModelAttribute("Bonus") Bonus bonus) {

        TbCfUserInfo info = user.user();

        Optional<String> userIdOptional = bonus.userId();

        double amount = bonus.getAmount();

        if (userIdOptional.isPresent()) {

            Optional<TbCfUserInfo> optionalUser = userRepository.findById(userIdOptional.get());

            if (optionalUser.isPresent()) {

                TbCfUserInfo user = optionalUser.get();

                Post post = bonus.getPost();

                Optional<Post> postOptional = post == null ? Optional.empty() : postRepository.findById(post.getId());

                String productSharer = bonus.getProductSharer();

                Optional<TbCfUserInfo> sharer = Optional.empty();

                if( productSharer != null){
                    sharer = userRepository.findByCode(productSharer);
                }


                if (user.invited()) {
                    double v = amount * 10 / 100;
                    bonus.setAmount(v);
                    bonus.setUserInfo(user);
                    bonus.setPercentage(10);
                    bonus = repository.save(bonus);
                    if( user.hasFcm() ){
                        sendNotification(user.getFcm(),"Bonus alert !!",user.display()+", You received bonus of $"+formatter.format(v)+" in your account");
                    }
                }


                TbCfUserInfo bonusInc = runBonusInc(user, amount, 5,false,bonus.getOrderId());
                runBonusInc(sharer.orElseGet(() -> postOptional.isPresent() ? postOptional.get().getRealUser() : bonusInc), amount, 5,postOptional.isPresent() || sharer.isPresent(),bonus.getOrderId());
                //runBonusInc(bonusInc, amount, 0);
            }
        }

        return bonus;
    }

    private TbCfUserInfo runBonusInc(TbCfUserInfo user, double amount, int percent,boolean direct,String orderId) {
        if (user == null) return null;
        Optional<Network> userCode = networkRepository.findByNetworkInfoCode(user.getCode());
        if (userCode.isPresent() || direct) {
            TbCfUserInfo userInfo = direct ? user : userCode.get().getUserInfo();
            Bonus bonus = new Bonus();
            bonus.setUserInfo(userInfo);
            double v = amount * percent / 100;
            bonus.setAmount(v);
            bonus.setPercentage(percent);
            bonus.setOrderId(orderId);
            if (userInfo.invited()){
                repository.save(bonus);
                bonus = repository.save(bonus);
                if( userInfo.hasFcm() ){
                    sendNotification(userInfo.getFcm(),"Bonus alert !!",userInfo.display()+", You received bonus of $"+formatter.format(v)+" in your account");
                }
            }
            return userInfo;
        }

        return null;
    }
}
