package com.example.afrishop_v3.controllers;

import com.example.afrishop_v3.models.*;
import com.example.afrishop_v3.repository.*;
import com.example.afrishop_v3.security.services.AuthenticationUser;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.google.api.client.util.Base64;
import com.google.api.client.util.StringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.domain.PageRequest;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

@RestController
@RequestMapping("/discover/post")
public class PostController {
    @Value("${upload.api}")
    private String api;

    private final PostRepository repository;
    private final LikeRepository likeRepository;
    private final CommentRepository commentRepository;
    private final PostTagRepository postTagRepository;
    private final ContentRepository contentRepository;
    private final UserRepository userRepository;
    private final VisitRepository visitRepository;
    private final BonusRepository bonusRepository;
    private final HashtagRepository hashtagRepository;
    private final TbCfStationItemRepository itemRepository;
    private final TagRepository tagRepository;
    private final PostHashtagRepository postHashtagRepository;
    private final AuthenticationUser user;


    public PostController(PostRepository repository, LikeRepository likeRepository, CommentRepository commentRepository, PostTagRepository postTagRepository, ContentRepository contentRepository, UserRepository userRepository, VisitRepository visitRepository, BonusRepository bonusRepository, HashtagRepository hashtagRepository, TbCfStationItemRepository itemRepository, TagRepository tagRepository, PostHashtagRepository postHashtagRepository, AuthenticationUser user) {
        this.repository = repository;
        this.likeRepository = likeRepository;
        this.commentRepository = commentRepository;
        this.postTagRepository = postTagRepository;
        this.contentRepository = contentRepository;
        this.userRepository = userRepository;
        this.visitRepository = visitRepository;
        this.bonusRepository = bonusRepository;
        this.hashtagRepository = hashtagRepository;
        this.itemRepository = itemRepository;
        this.tagRepository = tagRepository;
        this.postHashtagRepository = postHashtagRepository;
        this.user = user;
    }

    @GetMapping("/listPosts")
    public List<Post> postList(@RequestParam(value = "userId", required = false) String id, @RequestParam(value = "pageNo") Integer pageNo, @RequestParam(value = "pageSize") Integer pageSize) {
        //Optional<User> byId = userRepository.findById(id);
        //if( !byId.isPresent() ) return new ArrayList<>();
        List<Post> postList = repository.findAllByOrderByCreateDateDesc(PageRequest.of(pageNo, pageSize)).toList();
        return id == null ? postList : postList(postList, id);
    }

    @GetMapping("/listPostsByHashtag/{tag}")
    public List<Post> postListByTag(@PathVariable(value = "tag") String tag, @RequestParam(value = "pageNo") Integer pageNo, @RequestParam(value = "pageSize") Integer pageSize) {
        //Optional<User> byId = userRepository.findById(id);
        //if( !byId.isPresent() ) return new ArrayList<>();

        Optional<Hashtag> byName = hashtagRepository.findByName(tag);
        if (!byName.isPresent()) return new ArrayList<>();

        return repository.findAllByOrderByCreateDateDesc(byName.get(), PageRequest.of(pageNo, pageSize)).toList();
    }

    @GetMapping("/listPostsLiked/{userId}")
    public List<Post> postListLiked(@PathVariable(value = "userId") String id, @RequestParam(value = "pageNo") Integer pageNo, @RequestParam(value = "pageSize") Integer pageSize) {
        Optional<TbCfUserInfo> byId = userRepository.findById(id);
        if (!byId.isPresent()) return new ArrayList<>();
        List<Post> postList = repository.findAllByOrderByCreateDateDesc(byId.get(), PageRequest.of(pageNo, pageSize)).toList();
        return id == null ? postList : postList(postList, id);
    }


    @GetMapping("/visit/{postId}")
    public Visit visitPage(@PathVariable(value = "postId") String id) {
        Optional<Post> byId = repository.findById(id);
        if (byId.isPresent()) {
            Post post = byId.get();
            return visitPost(post);
        }
        return null;
    }


    private Visit visitPost(Post post) {
        Visit visit = new Visit();
        visit.setPost(post);
        return visitRepository.save(visit);
    }


    @PostMapping(value = "/repost/{userId}/{postId}")
    public Optional<Post> saveRePost(@PathVariable("userId") String id, @PathVariable("postId") String postId) {
        Optional<TbCfUserInfo> byId = userRepository.findById(id);
        Optional<Post> optional = repository.findById(postId);
        if (byId.isPresent() && optional.isPresent()) {
            Post post = optional.get();
            List<PostHashtag> hashtagList = new ArrayList<>(post.hashtagList);
            List<Content> contentList = new ArrayList<>(post.contentList);
            Post nPost = new Post();
            nPost.setUser(byId.get());
            nPost.setTitle(post.getTitle());
            if(post.getDescription() != null) nPost.setDescription(post.getDescription());
            nPost.setCategory(post.getCategory());
            nPost = repository.save(nPost);
            Post finalNPost = nPost;
            hashtagList.forEach(f -> {
                PostHashtag hashtag = new PostHashtag();
                hashtag.setPost(finalNPost);
                hashtag.setHashtag(f.getHashtag());
                postHashtagRepository.save(hashtag);
            });
            contentList.forEach(f -> {
                List<ContentTag> tagList = new ArrayList<>(f.contentTags);
                Content content = new Content();
                content.setPost(finalNPost);
                content.setThumbnail(f.getThumbnail());
                content.setImage(f.isImage());
                content.setSize(f.getSize());
                content.setContent(f.getContent());
                content = contentRepository.save(content);
                Content finalContent = content;
                tagList.forEach(x -> {
                    ContentTag contentTag = new ContentTag();
                            contentTag.setContent(finalContent);
                            contentTag.setX(x.getX());
                            contentTag.setY(x.getY());
                            contentTag.setTag(x.getTag());
                            postTagRepository.save(contentTag);
                        });

            });
            return Optional.of(nPost);

        }
        return Optional.empty();
    }



    private List<Post> postList(List<Post> postList, String userId) {
        for (Post post : postList) {
            post.liked = likeRepository.existsByUserInfo_UserIdAndPost_Id(userId, post.getId());
        }

        return postList;
    }

    @GetMapping("/listPostsByUser/{userId}")
    public List<Post> postListBy(@PathVariable(value = "userId") String id, @RequestParam(value = "pageNo") Integer pageNo, @RequestParam(value = "pageSize") Integer pageSize) {
        List<Post> postList = repository.findAllByUserInfoUserIdOrderByCreateDateDesc(id, PageRequest.of(pageNo, pageSize)).toList();
        return this.postList(postList, id);
    }

    @GetMapping("/searchPost/{query}")
    public List<Post> postListSearchBy(@PathVariable(value = "query") String query, @RequestParam(value = "pageNo") Integer pageNo, @RequestParam(value = "pageSize") Integer pageSize) {
        return repository.findAllByDescriptionContainingOrTitleContainingOrUserInfoNickContaining(query,query,query, PageRequest.of(pageNo, pageSize)).toList();
        //return this.postList(postList, id);
    }

    @GetMapping("/delete/post/{postId}")
    public String del(@PathVariable("postId") String id) {
        try {

            likeRepository.removeByPost_Id(id);
            commentRepository.removeByPost_Id(id);
            postTagRepository.removeByPost_Id(id);
            visitRepository.removeByPost_Id(id);
            bonusRepository.removeByPost_Id(id);
            contentRepository.removeByPost_Id(id);
            repository.deleteById(id);
        } catch (Exception e) {
            System.out.println(e.getMessage());
        }
        return "deleted";
    }

    @Transactional()
    @PostMapping("/upload")
    public List<String> handleFileUpload(@RequestParam("files") MultipartFile[] files, @RequestParam("thumbs") MultipartFile[] thumbs, @ModelAttribute("Post") Post ePost,
                                         RedirectAttributes redirectAttributes) throws IOException {
        Post post = new Post();
        TbCfUserInfo user = this.user.user();
        post.setUser(user);
        post.setDescription(ePost.getDescription());
        post.setTitle(ePost.getTitle());
        //repository.save(post);
        List<String> list = new ArrayList<>();
        List<String> thumbList = new ArrayList<>();

        if (thumbs != null) {
            for (MultipartFile file : thumbs) {
                StringBuilder sb = new StringBuilder();
                sb.append("data:image/png;base64,");
                sb.append(StringUtils.newStringUtf8(Base64.encodeBase64(file.getBytes())));
//                Path store = storageService.store(file);
//                String string = MvcUriComponentsBuilder.fromMethodName(FileUploadController.class,
//                        "serveFile", store.getFileName().toString()).build().toUri().toString();
                String string = UploadController.sendFile(sb.toString(), api);
                thumbList.add(string);
            }
        }
        if (files != null) {
            for (MultipartFile file : files) {
                StringBuilder sb = new StringBuilder();
                sb.append("data:image/png;base64,");
                sb.append(StringUtils.newStringUtf8(Base64.encodeBase64(file.getBytes())));
//                Path store = storageService.store(file);
//                String string = MvcUriComponentsBuilder.fromMethodName(FileUploadController.class,
//                        "serveFile", store.getFileName().toString()).build().toUri().toString();
                String string = UploadController.sendFile(sb.toString(), api);
                list.add(string);
                Content content = new Content();
                content.setPost(post);
                content.setImage(true);
               // content.path = store.toFile().getAbsolutePath();
                content.setContent(string);
                content.setSize(file.getSize());
                post.contentList.add(content);
                //contentRepository.save(content);
            }
        }

        ObjectMapper mapper = new ObjectMapper();
        List<JsonTag> tags = null;
        List<Hashtag> hashtags = null;
        try {
            tags = mapper.readValue(ePost.getTags(), new TypeReference<List<JsonTag>>() {
            });
            hashtags = mapper.readValue(ePost.getHashtags(), new TypeReference<List<Hashtag>>() {
            });
        } catch (JsonProcessingException e) {
            System.out.println(e.getMessage());
        }

        if (hashtags != null) {
            for (Hashtag hashtag : hashtags) {
                Optional<Hashtag> byName = hashtagRepository.findByName(hashtag.getName());
                hashtag = hashtagRepository.existsByName(hashtag.getName()) && byName.isPresent() ?
                        byName.get() :
                        hashtag;
                PostHashtag postHashtag = new PostHashtag();
                postHashtag.setHashtag(hashtag);
                postHashtag.setPost(post);
                if( !postHashtagRepository.existsByHashtagNameAndPostId(hashtag.getName(),post.getId()) ) {
                    post.hashtagList.add(postHashtag);
                }
            }
        }

        try {

            if (tags != null) {
                int index = 0;
                int vid = 0;
                for (JsonTag jsonTag : tags) {
                    if (post.contentList.size() > index) {
                        Content content = post.contentList.get(index);
                        content.setImage(jsonTag.isImage());
                        if (!jsonTag.isImage()) {
                            if (vid < thumbList.size()) {
                                String s = thumbList.get(vid);
                                content.setThumbnail(s);
                                vid++;
                            }
                        }
                        //contentRepository.save(content);
                        for (Position position : jsonTag.list) {

                            Tag tag = position.getTag();

                            TbCfStationItem item = tag.getStationItem();

                            Optional<TbCfStationItem> itemOptional = itemRepository.findById(item.getItemId());

                            if (itemOptional.isPresent()) {
                                item = itemOptional.get();
                            } else {
                                //item = itemRepository.save(item);
                            }
                            tag.setStationItem(item);


                            ContentTag contentTag = new ContentTag();
                            if (!tagRepository.existsByTagName(tag.getTagName())) {
                                // tagRepository.save(tag);
                            } else {
                                tag = tagRepository.findByTagName(tag.getTagName());
                            }
                            contentTag.setTag(tag);
                            contentTag.setX(position.getX());
                            contentTag.setY(position.getY());
                            contentTag.setContent(content);
                            content.contentTags.add(contentTag);
                        }
                        index++;
                    }
                }
            }
        }catch (Exception e){
            System.out.println(e.getMessage());
        }
        //redirectAttributes.addFlashAttribute();

        try {
            repository.save(post);
        }catch (Exception e){
            System.out.println(e.getMessage());
        }

        return list;
    }

}
